/************************************************************************************************
** File:        main.c
** Description: Implementation of sample project to blink LEDs
** Version:		1.0
**
*
** Copyright 2017 Zilog Inc. ALL RIGHTS RESERVED.
*
*************************************************************************************************
* The source code in this file was written by an authorized Zilog employee or a licensed 
* consultant. The source code has been verified to the fullest extent possible. 
*
* Permission to use this code is granted on a royalty-free basis. However, users are cautioned to
* authenticate the code contained herein. 
* 
* ZILOG DOES NOT GUARANTEE THE VERACITY OF THIS SOFTWARE; ANY SOFTWARE CONTAINED HEREIN IS
* PROVIDED "AS IS." NO WARRANTIES ARE GIVEN, WHETHER EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
* IMPLIED WARRANTIES OF FITNESS FOR PARTICULAR PURPOSE OR MERCHANTABILITY. IN NO EVENT WILL ZILOG
* BE LIABLE FOR ANY SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OR ANY LIABILITY IN TORT,
* NEGLIGENCE, OR OTHER LIABILITY INCURRED AS A RESULT OF THE USE OF THE SOFTWARE, EVEN IF ZILOG 
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. ZILOG ALSO DOES NOT WARRANT THAT THE USE 
* OF THE SOFTWARE, OR OF ANY INFORMATION CONTAINED THEREIN WILL NOT INFRINGE ANY PATENT, 
* COPYRIGHT, OR TRADEMARK OF ANY THIRD PERSON OR ENTITY.

* THE SOFTWARE IS NOT FAULT-TOLERANT AND IS NOT DESIGNED, MANUFACTURED OR INTENDED FOR USE IN 
* CONJUNCTION WITH ON-LINE CONTROL EQUIPMENT, IN HAZARDOUS ENVIRONMENTS, IN APPLICATIONS 
* REQUIRING FAIL-SAFE PERFORMANCE, OR WHERE THE FAILURE OF THE SOFTWARE COULD LEAD DIRECTLY TO 
* DEATH, PERSONAL INJURY OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE (ALL OF THE FOREGOING, 
* "HIGH RISK ACTIVITIES"). ZILOG SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY TO HIGH 
* RISK ACTIVITIES.
*
************************************************************************************************/


#include "Z32F0642.h"
#include "timer.h"


/*********************************************************************
 *	Init_SysClock
 *	
 *	Initializes System Clock using external Oscillator with PLL to 
 *	run at 48Mhz. The PC12, PC13 are configured for the XTAL alternative
 *  function by default. 
 *	
 *	Requires:
 *		Nothing
 *	Returns: 
 *		Nothing
 **********************************************************************/
 
void Init_SysClock(void)
{
    uint32_t c;
    
    //Enable PC port for XIN/XOUT
    SCU->PER1 |= SCU_PER1_GPIOC_Msk;
    SCU->PCER1 |= SCU_PCER1_GPIOC_Msk;

    //Enable PD port for SXIN/SXOUT
    SCU->PER1 |= SCU_PER1_GPIOD_Msk;
    SCU->PCER1 |= SCU_PCER1_GPIOD_Msk;
    
    PORTEN->EN = 0x15;
    PORTEN->EN = 0x51;

        // Connect AF 3 for XIN/XOUT
    PCC->MR |= (3 << PCC_MR_P12_Pos) | (3 << PCC_MR_P13_Pos);
        //Analog
    PCC->CR |= (3 << PCC_CR_P12_Pos) | (3 << PCC_CR_P13_Pos);
    
        //Connect AF 3 for SXIN/SXOUT
    PCD->MR |= (3 << PCD_MR_P2_Pos) | (3 << PCD_MR_P3_Pos);
        //Analog
    PCC->MR |= (3 << PCD_CR_P2_Pos) | (3 << PCD_CR_P3_Pos);
    
    // Enable Monitoring:
    SCU->CMR |= (1 << SCU_CMR_SOSCMNT_Pos) | (1 << SCU_CMR_MOSCMNT_Pos);
    
    // Enable clocks:
   
    SCU->CSCR = (2 << SCU_CSCR_SOSCCON_Pos) |
                (2 << SCU_CSCR_LSICON_Pos) |
                (2 << SCU_CSCR_HSICON_Pos) |
                (3 << SCU_CSCR_MOSCCON_Pos);  //Enable 16Mhz divide by 2 (8 Mhz)
    
    c=100;  //Make sure it times out if there is no external crystal
    while (!(SCU->CMR & SCU_CMR_MOSCSTS_Msk) && c-- > 0) ;  //check for oscillation
    for (c=0;c<200;++c) ;  //5ms for MOSC to stablize
  
    c=100;
    while (!(SCU->CMR & SCU_CMR_SOSCSTS_Msk) && c-- > 0) ;  //check for oscillation
    // Wait for stabilize MOSC: 5ms, SOSC 10ms @ 40Khz
    for (c=0;c<200;++c) ;  //5ms more for SOSC

   // Setup self calibration for HSI 
    FM->MR = 0xA5;
    FM->MR = 0x5A;
    
    SCU->BISCCON = (0x6A << SCU_BISCCON_XTAL_COMP_Pos);  //0x6A count for 32.768Khz crystal period
    SCU->HSIOSCTRIM |= (1u << SCU_HSIOSCTRIM_BISCON_Pos) | (1u << SCU_HSIOSCTRIM_REFSEL_Pos);  //Self cal enabled, using external SOSC
    
    FM->MR=0;           //reset the flash controller
    
    //Need to wait for self-cal - 8 cycles
    
    for (c=0;c<10;++c) ;

    //Select the clock Here
    SCU->SCCR = (4 << SCU_SCCR_MCLKSEL_Pos);

    

}	

/*********************************************************************
 *	ClockOut
 *	
 *	Initializes Clock Out for montoring Clock system.
 *	
 *	Requires:
 *		Nothing
 *	Returns: 
 *		Nothing
 **********************************************************************/
void ClockOut(void)
{
    uint32_t c;
    
    PORTEN->EN = 0x15;
    PORTEN->EN = 0x51;
    
    c = PCC->MR;
    c &= ~PCC_MR_P9_Msk;
    c |= (1 << PCC_MR_P9_Pos);   //Set to Alt function 01
    PCC->MR = c;
    
    PCC->CR &= ~PCC_CR_P9_Msk;      //Put to output
    
    PORTEN->EN = 0;
    
    SCU->COR = (1 << SCU_COR_CLKOEN_Pos) | (0 << SCU_COR_CLKODIV_Pos);
}

/*********************************************************************
 *	Init_LEDs
 *	
 *	Initializes the GPIO pins that control the LEDs (PB0-PB5)
 *	and keep LEDs off until the first timer IRQ fires
 *	Requires:
 *		Nothing
 *	Returns: 
 *		Nothing
 **********************************************************************/
void Init_LEDs(void)
{
	// Set PB0-PB5 to output and keep off
	// Make sure they are off
	SCU->PER1 |= (1 << SCU_PER1_GPIOB_Pos);			//Enable PB port
	SCU->PCER1 |= (1 << SCU_PCER1_GPIOB_Pos);			//Enable PB port clock
	
	PB->BCR = 0x3F;                 //PB0-5 off
	
	PORTEN->EN = 0x15;						//Unlock PCU
	PORTEN->EN = 0x51;
	
	PCB->MR &= 0xFFFFF000;				//GPIO mode for (PB0-5)
	PCB->CR &= 0xFFFFF000;				//Output
	
	PORTEN->EN = 0;								// Lock PCU
}

/*********************************************************************
 *	main()
 *	
 *	Application Entry. Initializes periphreals and then runs an 
 *	infinite loop, letting interrupts turn LEDs on and off
 *	
 *	Requires:
 *		Nothing
 *	Returns: 
 *		Nothing
 **********************************************************************/
int main(void)
{
	Init_SysClock();
	ClockOut();
	
	Init_LEDs();
	Init_Timer();
	
	while (1)
	{
		__NOP();			//Do nothing, let the interrupt handle it
	}
}

